"""
Author: Sanket S. Gadge
Contact: gadgesanket75@gmail.com
Date modified: 01-03-2021
Description: This script extracts Aadhaar information from the image uploaded by consumer. The result is returned
as a json object.
"""

import json

import cv2
import pytesseract
import regex as re

# import easyocr
# READER = easyocr.Reader(['en'])


class Aadhar_Info_Extractor:
    def __init__(self):
        # self.reader = reader
        self.extracted = {}

    def find_adhar_number(self, ocr_text):
        """Function to find adhar number inside the image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        str: 12 digit aadhaar number
        """
        adhar_number_patn = '[0-9]{4}\s[0-9]{4}\s[0-9]{4}'
        match = re.search(adhar_number_patn, ocr_text)
        if match:
            return match.group()

    def find_name(self, ocr_text):
        """Function to find adhar name inside the image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        str: name on the aadhar card
        """
        adhar_name_patn = r'\b[A-Z][a-z]+\s[A-Z][a-z]+\s[A-Z][a-z]+$'
        split_ocr = ocr_text.split('\n')
        for ele in split_ocr:
            match = re.search(adhar_name_patn, ele)
            if match:
                return match.group()

    def find_dob(self, ocr_text):
        """Function to find date of birth inside the image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        str: Date of birth
        """
        dob_patn = '\d+[-/]\d+[-/]\d+'
        yob_patn = '[0-9]{4}'
        DateOfBirth = ''
        if 'DOB' in ocr_text:
            match = re.search(dob_patn, ocr_text)
            DateOfBirth = match.group()
        if 'Year of Birth' in ocr_text:
            match = re.search(yob_patn, ocr_text)
            DateOfBirth = match.group()
        return DateOfBirth

    def find_gender(self, ocr_text):
        """Function to find Gender inside the image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        str: Gender
        """
        if 'Male' in ocr_text or 'MALE' in ocr_text:
            GENDER = 'Male'
        elif 'Female' in ocr_text or 'FEMALE' in ocr_text:
            GENDER = 'Female'
        else:
            GENDER = 'NAN'
        return GENDER

    def find_address(self, backimg):
        """Function to find address inside the image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        str: address on the aadhaar card
        """
        ocr_text = pytesseract.image_to_string(
        backimg, config='-l eng --psm 6 --oem 3 ')
        address_start = ocr_text.find('Address')
        address = ocr_text[address_start:]
        pinpatn = r'[0-9]{6}'
        address_end = 0
        pinloc = re.search(pinpatn, address)
        if pinloc:
            address_end = pinloc.end()
        else:
            print('Pin code not found in address')
            address = re.sub('\n', '', address[:address_end])
        address = address.split(':')[1]
        return address

    def info_extractor(self, front_image, back_image):
        """Function to extract information from the aadhaar card image

        Args:
        ocr_text (list): text from the ocr

        Returns:
        json: Data extracted from Adhar photograph
        """
        self.fimage = front_image
        self.bimage = back_image
        self.Name = 'NAN'
        self.Gender = 'NAN'
        self.DateOB = 'NAN'
        self.Aadhar_No = 'NAN'
        self.Address = 'NAN'
        # front image
        img = cv2.imread(self.fimage)
        # OCR_text = self.reader.readtext(img, detail=0,width_ths=0.9)
        OCR_text = pytesseract.image_to_string(img)

        self.Aadhar_No = self.find_adhar_number(OCR_text)
        self.Name = self.find_name(OCR_text)
        self.DateOB = self.find_dob(OCR_text)
        self.Gender = self.find_gender(OCR_text)

        # back image
        backimg = cv2.imread(self.bimage)
        backimg = cv2.cvtColor(backimg, cv2.COLOR_BGR2GRAY)
        self.Address = self.find_address(backimg)

        self.extracted = {
        'Aadhar_number': self.Aadhar_No,
        'Name': self.Name,
        'Gender': self.Gender,
        'DOB': self.DateOB,
        'Address': self.Address
        }

        # return self.extracted
        return json.dumps(self.extracted)



"""
    Author: Sanket S. Gadge
    Contact: gadgesanket75@gmail.com
    Date modified: 01-03-2021
    Description: This script extracts Pan information from the image uploaded by consumer. The result is returned
                as a json object.
"""

import cv2
import regex as re
import json
import easyocr


class Pan_Info_Extractor:
    def __init__(self):
        try:
            import warnings
            warnings.filterwarnings('ignore')
            self.reader = easyocr.Reader(['en'])
        except ImportError:
            print('No module named: easyocr')
        self.datepatn = r'\d+[-/]\d+[-/]\d+'
        self.panpatn = r'([A-Z]){5}([0-9]){4}([A-Z]){1}'
        self.namepatn = r'([A-Z]+)\s([A-Z]+)\s([A-Z]+)'
        self.fnamepatn = r'([A-Z]+)\s+?'
        self.godpatn = r'([A-Z]+)\s([A-Z]+)\s([A-Z]+)$|([A-Z]+)\s([A-Z]+)$'

    def info_extractor(self, image_file):
        """Function to extract information from the pan card image

        Args:
                ocr_text (list): text from the ocr

        Returns:
                json: Data extracted from pan card image
        """
        self.PAN = 'NAN'
        self.Name = 'NAN'
        self.FatherName = 'NAN'
        self.DOB = 'NAN'
        self.image = image_file
        img = cv2.imread(self.image)
        OCR_text = self.reader.readtext(img, detail=0, width_ths=0.9)
        # print(OCR_text)

        gov = [i for i, txt in enumerate(OCR_text) if 'GOVT' in txt][0]
        OCR_text = OCR_text[gov + 1:]
        temp = []
        for text in OCR_text:
            name = re.search(self.godpatn, text)
            # print(name.group())
            if name:
                temp.append(name.group())

        if self.Name == 'NAN':
            self.Name = temp[0]
        if self.FatherName == 'NAN':
            self.FatherName = temp[1]

        for text in OCR_text:
            if self.PAN == 'NAN' and re.search(self.panpatn, text):
                self.PAN = re.search(self.panpatn, text).group()
                break

        for text in OCR_text:
            if self.DOB == 'NAN' and re.search(self.datepatn, text):
                self.DOB = re.search(self.datepatn, text).group()
                break

        extracted = {
            'Pan_number': self.PAN,
            'Name': self.Name,
            'Father_Name': self.FatherName,
            'DOB': self.DOB
        }

        return json.dumps(extracted)



from setuptools import setup

setup(
    name='pan_aadhar_ocr',
    packages=['pan_aadhar_ocr'],
    include_package_data=True,
    version='1.0.3',
    license='License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
    description='Extract information from the pan and aadhar card image',
    long_description=open('README.md').read(),	
    long_description_content_type="text/markdown",
    author='Sanket Gadge',
    author_email='gadgesanket75@gmail.com',
    url='https://github.com/sanket-eazr/Pan-Aadhaar-OCR-With-API',
    download_url='https://github.com/sanket-eazr/Pan-Aadhaar-OCR-With-API.git',
    keywords=['ocr optical character recognition deep learning neural network'],
    install_requires=[
    	"regex==2020.10.15",
	"pytesseract==0.3.7",
	"easyocr==1.2.4",
    ],
)
